//
//  CarouselViewController.m
//  CellCarousel
//
//  Created by Dean on 17/03/16.
//  Copyright © 2015 invstr. All rights reserved.
//

#import "CarouselViewController.h"

@interface CarouselViewController() <IGGridViewDataSource, IGGridViewDelegate>
{
    IGGridView* gridView;
    IGGridViewSingleRowSingleFieldDataSourceHelper* datasource;
    
    CGFloat cellSpacing;
}

@property (nonatomic, strong) NSArray* dataArray;

@end

@implementation CarouselViewController

-(void)viewDidLoad
{
    [super viewDidLoad];
    
    NSMutableArray* dataSource = [[NSMutableArray alloc] init];
    for (int i = 0; i <= 20; i++)
    {
        CarouselDataItem* dataItem = [[CarouselDataItem alloc] init];
        dataItem.title = @"test";
        [dataSource addObject:dataItem];
    }
    
    self.dataArray = [[NSArray alloc] initWithArray:[dataSource copy]];
    [self initialize:100];
    dataSource = nil;
}

-(void)viewDidLayoutSubviews
{
    [super viewDidLayoutSubviews];
    
    CGFloat cellSize = gridView.rowHeight;
    CGFloat inset = self.view.frame.size.width / 2 - cellSize / 2 + cellSpacing;
    CGFloat top = 0;
    gridView.contentInset = UIEdgeInsetsMake(top, inset, 0, inset);
}

-(void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
}

#pragma mark - Private Methods

-(void)initialize:(NSInteger)originY
{
    if (dataSource == nil)
    {
        return;
    }
    
    cellSpacing = (self.view.frame.size.width / 11);
    
    // update the current view controller's frame
    CGRect frame = CGRectMake(0, originY, self.view.frame.size.width, self.view.frame.size.height);
    self.view.frame = frame;
    
    // initialize the gridview
    gridView = [[IGGridView alloc]initWithFrame:CGRectMake(0, 0, self.view.frame.size.width, self.view.frame.size.height) style:IGGridViewStyleDefault];
    gridView.autoresizingMask = UIViewAutoresizingFlexibleHeight | UIViewAutoresizingFlexibleWidth;
    [self.view addSubview:gridView];
    
    CGFloat rowHeight = .30;
    CGFloat base = MIN(self.view.frame.size.width, self.view.frame.size.height);
    gridView.rowHeight = base * rowHeight;
    
    // set initial settings
    gridView.clipsToBounds = NO;
    gridView.headerHeight = 0;
    gridView.selectionType = IGGridViewSelectionTypeNone;
    gridView.rowSeparatorHeight = 0;
    gridView.emptyRows = NO;
    gridView.alwaysBounceVertical = NO;
    gridView.allowHorizontalBounce = YES;
    gridView.columnWidth = [IGColumnWidth CreateWithFixedWidth:gridView.rowHeight];
    gridView.columnSpacing = -cellSpacing;
    gridView.dataSource = self;
    gridView.delegate = self;
    
    [gridView setShowsHorizontalScrollIndicator:NO];
    [gridView setShowsVerticalScrollIndicator:NO];
    [gridView registerClass:[CarouselCell class] forCellReuseIdentifier:@"image"];
}

-(void)didDeselectRowAtIndexPath:(UITapGestureRecognizer*)gesture
{
    
}

#pragma mark - IGGridView Methods

-(NSInteger)numberOfFixedRowsInGridView:(IGGridView *)gridView
{
    return 0;
}

-(NSInteger)gridView:(IGGridView*)gridView numberOfRowsInSection:(NSInteger)section
{
    return 1;
}

-(NSInteger)numberOfColumnsInGridView:(IGGridView*)gridView
{
    return [self.dataArray count];
}

-(IGGridViewCell*)gridView:(IGGridView*)gridView cellAt:(IGCellPath*)path
{
    CarouselCell* cell = [[CarouselCell alloc]initWithReuseIdentifier:@"carouselCell"];
    cell.tag = path.columnIndex;
    cell.dataItem = [self.dataArray objectAtIndex:path.columnIndex];
    
    return cell;
}

@end

@interface CarouselCell()
{
    
}

@property CGFloat min;
@property CGFloat max;

@end

@implementation CarouselCell

-(id)init
{
    if ((self = [super init]))
    {
        
    }
    
    return self;
}

-(void)cellLocationInViewportChanged:(CGPoint)location
{
    @try
    {
        if (!self.dataItem)
        {
            return;
        }
        
        if (self.dataItem.isDummy)
        {
            return;
        }
        
        NSInteger screenHeight = [[UIScreen mainScreen] bounds].size.height;
        [self updateParameters];
        
        // initialize the background circle
        self.imageView.layer.transform = CATransform3DIdentity;
        self.imageView.layer.backgroundColor = [UIColor redColor];
        CALayer* layer = [self.imageView layer];
        [layer setMasksToBounds:YES];
        [layer setCornerRadius:(self.imageView.frame.size.height / 2)];
        
        // find the percent value from the middle
        CGFloat scale = [self centerLocation:location.x];
        scale = [self positionInBetween:scale min:self.min max:self.max];
        
        // find the cell's type
        BOOL isCenterCell = (scale > 1.46 && (location.x > .45f && location.x < .66f));
        BOOL isBehindTheCenterCell = (scale > 1 && scale < 1.46);
        
        // calculate the y position and transform the imageView's layer
        // CGPoint position = self.layer.position;
        // position.y = [self positionInBetween:scale min:70 max:70];

        // position.x = can be done here?
        
        // self.layer.position = position;
        
        [self applyScale:scale toView:self.imageView];
        
        if (isCenterCell)
        {
            self.imageView.layer.backgroundColor = [UIColor redColor];
            self.layer.zPosition = 1;
        }
        else
        {
            if (isBehindTheCenterCell)
            {
                self.layer.zPosition = 0.9;
            }
            else
            {
                self.layer.zPosition = 0;
            }
        }
        
        self.layer.shadowColor = [UIColor blackColor].CGColor;
        self.layer.shadowOffset = CGSizeMake(1, 1);
        self.layer.shadowOpacity = 0.5;
        self.layer.shadowRadius = 4;
    }
    @catch (NSException *exception)
    {
        
    }
}

-(void)applyScale:(CGFloat)percent toView:(UIView*)view
{
    CATransform3D transform = view.layer.transform;
    transform = CATransform3DScale(transform, percent, percent, percent);
    view.layer.transform = transform;
    
    self.percent = percent;
}

-(CGFloat)centerLocation:(CGFloat)val
{
    if (val >= .5)
    {
        val = 1 - val;
    }
    
    if (val <= .5)
    {
        val = val / .5;
    }
    
    return val;
}

-(CGFloat)positionInBetween:(CGFloat)scale min:(CGFloat)min max:(CGFloat)max
{
    return ((max - min) * scale) + min;
}

-(void)updateParameters
{
    self.min = 0.6;
    self.max = 1.5;
}

@end

@implementation CarouselDataItem

@end
